@@ -80,10 +80,12 @@ SymbolicFile::createSymbolicFile(MemoryBufferRef Object, file_magic Type,
     if (!Obj || !Context)
       return std::move(Obj);
 
-    ErrorOr<MemoryBufferRef> BCData =
+    Expected<MemoryBufferRef> BCData =
         IRObjectFile::findBitcodeInObject(*Obj->get());
-    if (!BCData)
+    if (!BCData) {
+      consumeError(BCData.takeError());
       return std::move(Obj);
+    }
 
     return IRObjectFile::create(
         MemoryBufferRef(BCData->getBuffer(), Object.getBufferIdentifier()),
