@@ -1,4 +1,4 @@
-//===--------------------AMDKernelCodeTUtils.cpp --------------------------===//
+//===- AMDKernelCodeTUtils.cpp --------------------------------------------===//
 //
 //                     The LLVM Compiler Infrastructure
 //
@@ -7,17 +7,21 @@
 //
 //===----------------------------------------------------------------------===//
 //
-//===----------------------------------------------------------------------===//
-//
 /// \file - utility functions to parse/print amd_kernel_code_t structure
 //
 //===----------------------------------------------------------------------===//
 
 #include "AMDKernelCodeTUtils.h"
 #include "SIDefines.h"
-#include <llvm/MC/MCParser/MCAsmLexer.h>
-#include <llvm/MC/MCParser/MCAsmParser.h>
-#include <llvm/Support/raw_ostream.h>
+#include "llvm/ADT/ArrayRef.h"
+#include "llvm/ADT/StringMap.h"
+#include "llvm/ADT/StringRef.h"
+#include "llvm/MC/MCParser/MCAsmLexer.h"
+#include "llvm/MC/MCParser/MCAsmParser.h"
+#include "llvm/Support/raw_ostream.h"
+#include <cassert>
+#include <cstdint>
+#include <utility>
 
 using namespace llvm;
 
@@ -62,7 +66,6 @@ static StringRef get_amd_kernel_code_t_FieldName(int index) {
   return get_amd_kernel_code_t_FldNames()[index + 1];
 }
 
-
 // Field printing
 
 static raw_ostream &printName(raw_ostream &OS, StringRef Name) {
@@ -82,9 +85,7 @@ static void printBitField(StringRef Name, const amd_kernel_code_t &c,
   printName(OS, Name) << (int)((c.*ptr >> shift) & Mask);
 }
 
-typedef void(*PrintFx)(StringRef,
-                       const amd_kernel_code_t &,
-                       raw_ostream &);
+using PrintFx = void(*)(StringRef, const amd_kernel_code_t &, raw_ostream &);
 
 static ArrayRef<PrintFx> getPrinterTable() {
   static const PrintFx Table[] = {
@@ -114,7 +115,6 @@ void llvm::dumpAmdKernelCode(const amd_kernel_code_t *C,
   }
 }
 
-
 // Field parsing
 
 static bool expectAbsExpression(MCAsmParser &MCParser, int64_t &Value, raw_ostream& Err) {
@@ -154,9 +154,8 @@ static bool parseBitField(amd_kernel_code_t &C, MCAsmParser &MCParser,
   return true;
 }
 
-typedef bool(*ParseFx)(amd_kernel_code_t &,
-                       MCAsmParser &MCParser,
-                       raw_ostream &Err);
+using ParseFx = bool(*)(amd_kernel_code_t &, MCAsmParser &MCParser,
+                        raw_ostream &Err);
 
 static ArrayRef<ParseFx> getParserTable() {
   static const ParseFx Table[] = {
