@@ -18,6 +18,8 @@
 
 namespace llvm {
 
+using namespace coverage;
+
 struct FileCoverageSummary;
 
 /// \brief A coverage printer for html output.
@@ -29,7 +31,8 @@ public:
   void closeViewFile(OwnedStream OS) override;
 
   Error createIndexFile(ArrayRef<std::string> SourceFiles,
-                        const coverage::CoverageMapping &Coverage) override;
+                        const coverage::CoverageMapping &Coverage,
+                        const CoverageFiltersMatchAll &Filters) override;
 
   CoveragePrinterHTML(const CoverageViewOptions &Opts)
       : CoveragePrinter(Opts) {}
@@ -38,6 +41,8 @@ private:
   void emitFileSummary(raw_ostream &OS, StringRef SF,
                        const FileCoverageSummary &FCS,
                        bool IsTotals = false) const;
+  std::string buildLinkToFile(StringRef SF,
+                              const FileCoverageSummary &FCS) const;
 };
 
 /// \brief A code coverage view which supports html-based rendering.
@@ -54,14 +59,11 @@ class SourceCoverageViewHTML : public SourceCoverageView {
 
   void renderViewDivider(raw_ostream &OS, unsigned ViewDepth) override;
 
-  void renderLine(raw_ostream &OS, LineRef L,
-                  const coverage::CoverageSegment *WrappedSegment,
-                  CoverageSegmentArray Segments, unsigned ExpansionCol,
-                  unsigned ViewDepth) override;
+  void renderLine(raw_ostream &OS, LineRef L, const LineCoverageStats &LCS,
+                  unsigned ExpansionCol, unsigned ViewDepth) override;
 
   void renderExpansionSite(raw_ostream &OS, LineRef L,
-                           const coverage::CoverageSegment *WrappedSegment,
-                           CoverageSegmentArray Segments, unsigned ExpansionCol,
+                           const LineCoverageStats &LCS, unsigned ExpansionCol,
                            unsigned ViewDepth) override;
 
   void renderExpansionView(raw_ostream &OS, ExpansionView &ESV,
@@ -75,7 +77,7 @@ class SourceCoverageViewHTML : public SourceCoverageView {
 
   void renderLineNumberColumn(raw_ostream &OS, unsigned LineNo) override;
 
-  void renderRegionMarkers(raw_ostream &OS, CoverageSegmentArray Segments,
+  void renderRegionMarkers(raw_ostream &OS, const LineCoverageStats &Line,
                            unsigned ViewDepth) override;
 
   void renderTitle(raw_ostream &OS, StringRef Title) override;
