@@ -131,6 +131,9 @@ TEST_F(TargetLibraryInfoTest, ValidProto) {
     "declare double @copysign(double, double)\n"
     "declare float @copysignf(float, float)\n"
     "declare x86_fp80 @copysignl(x86_fp80, x86_fp80)\n"
+    "declare double @cabs([2 x double])\n"
+    "declare float @cabsf([2 x float])\n"
+    "declare x86_fp80 @cabsl([2 x x86_fp80])\n"
     "declare double @cos(double)\n"
     "declare float @cosf(float)\n"
     "declare double @cosh(double)\n"
