@@ -89,8 +89,9 @@ public:
   bool needsToReserveAllocationSpace() override { return true; }
 
   void reserveAllocationSpace(uintptr_t CodeSize, uint32_t CodeAlign,
-			      uintptr_t DataSizeRO, uint32_t RODataAlign,
-                              uintptr_t DataSizeRW, uint32_t RWDataAlign) override {
+                              uintptr_t DataSizeRO, uint32_t RODataAlign,
+                              uintptr_t DataSizeRW,
+                              uint32_t RWDataAlign) override {
     ReservedCodeSize = CodeSize;
     ReservedDataSizeRO = DataSizeRO;
     ReservedDataSizeRW = DataSizeRW;
